@@ -1,46 +1,50 @@
 package CHI::Driver::Base::CacheContainer;
-use Moose;
+use Mouse;
+use Mouse::Util::TypeConstraints;
+use List::MoreUtils qw( all );
 use strict;
 use warnings;
 
 extends 'CHI::Driver';
 
+has '_contained_cache' => ( is => 'ro' );
+
 __PACKAGE__->meta->make_immutable();
 
 sub fetch {
     my ( $self, $key ) = @_;
 
-    return $self->{_contained_cache}->get($key);
+    return $self->_contained_cache->get($key);
 }
 
 sub store {
-    my ( $self, $key, $data ) = @_;
+    my $self = shift;
 
-    $self->{_contained_cache}->set( $key, $data );
+    return $self->_contained_cache->set(@_);
 }
 
 sub remove {
     my ( $self, $key ) = @_;
 
-    $self->{_contained_cache}->remove($key);
+    $self->_contained_cache->remove($key);
 }
 
 sub clear {
-    my ($self) = @_;
+    my $self = shift;
 
-    $self->{_contained_cache}->clear();
+    return $self->_contained_cache->clear(@_);
 }
 
 sub get_keys {
-    my ($self) = @_;
+    my $self = shift;
 
-    return $self->{_contained_cache}->get_keys();
+    return $self->_contained_cache->get_keys(@_);
 }
 
 sub get_namespaces {
-    my ($self) = @_;
+    my $self = shift;
 
-    return $self->{_contained_cache}->get_namespaces();
+    return $self->_contained_cache->get_namespaces(@_);
 }
 
 1;
@@ -51,11 +55,11 @@ __END__
 
 =head1 NAME
 
-CHI::Driver::Base::CacheContainer
+CHI::Driver::Role::CacheContainer
 
 =head1 DESCRIPTION
 
-Base class for CHI drivers with an internal '_contained_cache' slot that itself adheres to
+Role for CHI drivers with an internal '_contained_cache' slot that itself adheres to
 the Cache::Cache API, partially or completely.
 
 =head1 AUTHOR
