@@ -5,7 +5,9 @@ use CHI::Test;
 use CHI::Test::Logger;
 use CHI::Test::Util qw(cmp_bool is_between random_string skip_until);
 use CHI::Util qw(dump_one_line dp);
+use File::Temp qw(tempdir);
 use Module::Load::Conditional qw(can_load check_install);
+use Scalar::Util qw(weaken);
 use base qw(CHI::Test::Class);
 
 # Flags indicating what each test driver supports
@@ -24,8 +26,14 @@ sub standard_keys_and_values : Test(startup) {
 
 sub kvpair {
     my $self = shift;
+    my $count = shift || 1;
 
-    return ( $self->{keys}->{medium}, $self->{values}->{medium} );
+    return map {
+        (
+            $self->{keys}->{medium} .   ( $_ == 1 ? '' : $_ ),
+            $self->{values}->{medium} . ( $_ == 1 ? '' : $_ )
+          )
+    } ( 1 .. $count );
 }
 
 sub setup : Test(setup) {
@@ -118,6 +126,16 @@ sub test_simple : Test(1) {
     is( $cache->get( $self->{keys}->{medium} ), $self->{values}->{medium} );
 }
 
+sub test_driver_class : Tests(4) {
+    my $self  = shift;
+    my $cache = $self->{cache};
+
+    isa_ok( $cache, 'CHI::Driver' );
+    isa_ok( $cache, $cache->driver_class );
+    isa_ok( $cache, 'CHI::Driver::Wrapper' );
+    can_ok( $cache, 'get', 'set', 'remove', 'clear', 'expire' );
+}
+
 sub test_key_types : Tests {
     my $self  = shift;
     my $cache = $self->{cache};
@@ -293,21 +311,29 @@ sub test_expires_defaults : Test(4) {
     my $cache;
 
     my $set_and_confirm_expires_at = sub {
-        my ($expected_expires_at) = @_;
+        my ( $expected_expires_at, $desc ) = @_;
         my ( $key, $value ) = ( random_string(10), random_string(10) );
         $cache->set( $key, $value );
-        is( $cache->get_expires_at($key), $expected_expires_at );
+        is( $cache->get_expires_at($key), $expected_expires_at, $desc );
     };
 
     $cache = $self->new_cache( expires_in => 10 );
-    $set_and_confirm_expires_at->( $start_time + 10 );
+    $set_and_confirm_expires_at->(
+        $start_time + 10,
+        "after expires_in constructor option"
+    );
     $cache->expires_in(20);
-    $set_and_confirm_expires_at->( $start_time + 20 );
+    $set_and_confirm_expires_at->( $start_time + 20,
+        "after expires_in method" );
 
     $cache = $self->new_cache( expires_at => $start_time + 30 );
-    $set_and_confirm_expires_at->( $start_time + 30 );
+    $set_and_confirm_expires_at->(
+        $start_time + 30,
+        "after expires_at constructor option"
+    );
     $cache->expires_at( $start_time + 40 );
-    $set_and_confirm_expires_at->( $start_time + 40 );
+    $set_and_confirm_expires_at->( $start_time + 40,
+        "after expires_at method" );
 }
 
 sub test_expires_manually : Test(3) {
@@ -323,7 +349,7 @@ sub test_expires_manually : Test(3) {
     ok( !$cache->is_valid($key),    "invalid after expire ($desc)" );
 }
 
-sub test_expires_conditionally : Test(24) {
+sub test_expires_conditionally : Test(22) {
     my $self  = shift;
     my $cache = $self->{cache};
 
@@ -337,11 +363,8 @@ sub test_expires_conditionally : Test(24) {
             $cache->set( $key, $value );
             if ($separate_call) {
                 is( $cache->get($key), $value, "hit ($desc)" );
-                cmp_bool(
-                    $cache->expire_if( $key, $code ),
-                    $expect_expire ? 1 : 0,
-                    "expire_if ($desc)"
-                );
+                cmp_bool( $cache->expire_if( $key, $code ),
+                    $expect_expire, "expire_if ($desc)" );
             }
             else {
                 is(
@@ -618,6 +641,323 @@ sub test_multi_no_keys : Test(4) {
     lives_ok { $cache->remove_multi( [] ) } "remove_multi (no args)";
 }
 
+sub test_l1_cache : Test(228) {
+    my $self   = shift;
+    my @keys   = map { "key$_" } ( 0 .. 2 );
+    my @values = map { "value$_" } ( 0 .. 2 );
+    my ( $cache, $l1_cache );
+
+    my $test_l1_cache = sub {
+
+        is( $l1_cache->subcache_type, "l1_cache" );
+
+        # Get on cache should populate l1 cache
+        #
+        $cache->set( $keys[0], $values[0] );
+        $l1_cache->clear();
+        ok( !$l1_cache->get( $keys[0] ), "l1 miss after clear" );
+        is( $cache->get( $keys[0] ),
+            $values[0], "primary hit after primary set" );
+        is( $l1_cache->get( $keys[0] ), $values[0],
+            "l1 hit after primary get" );
+
+        # Primary cache should be reading l1 cache first
+        #
+        $l1_cache->set( $keys[0], $values[1] );
+        is( $cache->get( $keys[0] ),
+            $values[1], "got new value set explicitly in l1 cache" );
+        $l1_cache->remove( $keys[0] );
+        is( $cache->get( $keys[0] ), $values[0], "got old value again" );
+
+        $cache->clear();
+        ok( !$cache->get( $keys[0] ),    "miss after clear" );
+        ok( !$l1_cache->get( $keys[0] ), "miss after clear" );
+
+        # get_multi_* - one from l1 cache, one from primary cache, one miss
+        #
+        $cache->set( $keys[0], $values[0] );
+        $cache->set( $keys[1], $values[0] );
+        $l1_cache->remove( $keys[0] );
+        $l1_cache->set( $keys[1], $values[1] );
+        cmp_deeply(
+            $cache->get_multi_arrayref( [ $keys[0], $keys[1], $keys[2] ] ),
+            [ $values[0], $values[1], undef ],
+            "get_multi_arrayref"
+        );
+        cmp_deeply(
+            [ $cache->get_multi_array( [ $keys[0], $keys[1], $keys[2] ] ) ],
+            [ $values[0], $values[1], undef ],
+            "get_multi_array"
+        );
+        cmp_deeply(
+            $cache->get_multi_hashref( [ $keys[0], $keys[1], $keys[2] ] ),
+            {
+                $keys[0] => $values[0],
+                $keys[1] => $values[1],
+                $keys[2] => undef
+            },
+            "get_multi_hashref"
+        );
+
+        $self->_test_logging_with_l1_cache( $cache, $l1_cache );
+
+        $self->_test_common_subcache_features( $cache, $l1_cache );
+    };
+
+    # Test with current cache in primary position...
+    #
+    $cache = $self->new_cache( l1_cache => { driver => 'Memory' } );
+    $l1_cache = $cache->l1_cache;
+    isa_ok( $cache,    $self->testing_driver_class );
+    isa_ok( $l1_cache, 'CHI::Driver::Memory' );
+    $test_l1_cache->();
+
+    # and in l1 position
+    #
+    $cache = CHI->new(
+        driver   => 'Memory',
+        l1_cache => { $self->new_cache_options() }
+    );
+    $l1_cache = $cache->l1_cache;
+    isa_ok( $cache,    'CHI::Driver::Memory' );
+    isa_ok( $l1_cache, $self->testing_driver_class );
+    $test_l1_cache->();
+}
+
+sub test_mirror_cache : Test(206) {
+    my $self = shift;
+    my ( $cache, $mirror_cache );
+    my ( $key, $value, $key2, $value2 ) = $self->kvpair(2);
+
+    my $test_mirror_cache = sub {
+
+        is( $mirror_cache->subcache_type, "mirror_cache" );
+
+        # Get on either cache should not populate the other, and should not be able to see
+        # mirror keys from regular cache
+        #
+        $cache->set( $key, $value );
+        $mirror_cache->remove($key);
+        $cache->get($key);
+        ok( !$mirror_cache->get($key), "key not in mirror_cache" );
+
+        $mirror_cache->set( $key2, $value2 );
+        ok( !$cache->get($key2), "key2 not in cache" );
+
+        $self->_test_logging_with_mirror_cache( $cache, $mirror_cache );
+
+        $self->_test_common_subcache_features( $cache, $mirror_cache );
+    };
+
+    my $file_cache_options = sub {
+        my $root_dir =
+          tempdir( "chi-test-mirror-cache-XXXX", TMPDIR => 1, CLEANUP => 1 );
+        return ( driver => 'File', root_dir => $root_dir, depth => 3 );
+    };
+
+    # Test with current cache in primary position...
+    #
+    $cache = $self->new_cache( mirror_cache => { $file_cache_options->() } );
+    $mirror_cache = $cache->mirror_cache;
+    isa_ok( $cache,        $self->testing_driver_class );
+    isa_ok( $mirror_cache, 'CHI::Driver::File' );
+    $test_mirror_cache->();
+
+    # and in mirror position
+    #
+    $cache =
+      CHI->new( $file_cache_options->(),
+        mirror_cache => { $self->new_cache_options() } );
+    $mirror_cache = $cache->mirror_cache;
+    isa_ok( $cache,        'CHI::Driver::File' );
+    isa_ok( $mirror_cache, $self->testing_driver_class );
+    $test_mirror_cache->();
+}
+
+# Run logging tests for a cache with an l1_cache
+#
+sub _test_logging_with_l1_cache {
+    my ( $self, $cache ) = @_;
+
+    $cache->clear();
+    my $log = CHI::Test::Logger->new();
+    CHI->logger($log);
+    my ( $key, $value ) = $self->kvpair();
+
+    my $driver = $cache->short_driver_name;
+
+    my $miss_not_in_cache = 'MISS \(not in cache\)';
+    my $miss_expired      = 'MISS \(expired\)';
+
+    my $start_time = time();
+
+    $cache->get($key);
+    $log->contains_ok(
+        qr/cache get for .* key='$key', cache='$driver': $miss_not_in_cache/);
+    $log->contains_ok(
+        qr/cache get for .* key='$key', cache='.*l1.*': $miss_not_in_cache/);
+    $log->empty_ok();
+
+    $cache->set( $key, $value, 80 );
+    my $length = length($value);
+    $log->contains_ok(
+        qr/cache set for .* key='$key', size=$length, expires='1m20s', cache='$driver'/
+    );
+    $log->contains_ok(
+        qr/cache set for .* key='$key', size=$length, expires='1m20s', cache='.*l1.*'/
+    );
+    $log->empty_ok();
+
+    $cache->get($key);
+    $log->contains_ok(qr/cache get for .* key='$key', cache='.*l1.*': HIT/);
+    $log->empty_ok();
+
+    local $CHI::Driver::Test_Time = $start_time + 120;
+    $cache->get($key);
+    $log->contains_ok(
+        qr/cache get for .* key='$key', cache='$driver': $miss_expired/);
+    $log->contains_ok(
+        qr/cache get for .* key='$key', cache='.*l1.*': $miss_expired/);
+    $log->empty_ok();
+
+    $cache->remove($key);
+    $cache->get($key);
+    $log->contains_ok(
+        qr/cache get for .* key='$key', cache='$driver': $miss_not_in_cache/);
+    $log->contains_ok(
+        qr/cache get for .* key='$key', cache='.*l1.*': $miss_not_in_cache/);
+    $log->empty_ok();
+}
+
+sub _test_logging_with_mirror_cache {
+    my ( $self, $cache ) = @_;
+
+    $cache->clear();
+    my $log = CHI::Test::Logger->new();
+    CHI->logger($log);
+    my ( $key, $value ) = $self->kvpair();
+
+    my $driver = $cache->short_driver_name;
+
+    my $miss_not_in_cache = 'MISS \(not in cache\)';
+    my $miss_expired      = 'MISS \(expired\)';
+
+    my $start_time = time();
+
+    $cache->get($key);
+    $log->contains_ok(
+        qr/cache get for .* key='$key', cache='$driver': $miss_not_in_cache/);
+    $log->empty_ok();
+
+    $cache->set( $key, $value, 80 );
+    my $length = length($value);
+    $log->contains_ok(
+        qr/cache set for .* key='$key', size=$length, expires='1m20s', cache='$driver'/
+    );
+    $log->contains_ok(
+        qr/cache set for .* key='$key', size=$length, expires='1m20s', cache='.*mirror.*'/
+    );
+    $log->empty_ok();
+
+    $cache->get($key);
+    $log->contains_ok(qr/cache get for .* key='$key', cache='$driver': HIT/);
+    $log->empty_ok();
+
+    local $CHI::Driver::Test_Time = $start_time + 120;
+    $cache->get($key);
+    $log->contains_ok(
+        qr/cache get for .* key='$key', cache='$driver': $miss_expired/);
+    $log->empty_ok();
+
+    $cache->remove($key);
+    $cache->get($key);
+    $log->contains_ok(
+        qr/cache get for .* key='$key', cache='$driver': $miss_not_in_cache/);
+    $log->empty_ok();
+}
+
+# Run tests common to l1_cache and mirror_cache
+#
+sub _test_common_subcache_features {
+    my ( $self, $cache, $subcache ) = @_;
+    my ( $key, $value, $key2, $value2 ) = $self->kvpair(2);
+
+    for ( $cache, $subcache ) { $_->clear() }
+
+    # Test informational methods
+    #
+    ok( !$cache->is_subcache,             "is_subcache - false" );
+    ok( $subcache->is_subcache,           "is_subcache - true" );
+    ok( !defined( $cache->parent_cache ), "parent_cache - undef" );
+    is( $subcache->parent_cache, $cache, "parent_cache - defined" );
+    ok( !defined( $cache->subcache_type ), "subcache_type - undef" );
+    cmp_deeply( $cache->subcaches,    [$subcache], "subcaches - empty" );
+    cmp_deeply( $subcache->subcaches, [],          "subcaches - empty" );
+
+    # Test that sets and various kinds of removals and expirations are distributed to both
+    # the primary cache and the subcache
+    #
+    my ( $test_remove_method, $confirm_caches_empty,
+        $confirm_caches_populated );
+    $test_remove_method = sub {
+        my ( $desc, $remove_code ) = @_;
+        $desc = "testing $desc";
+
+        $confirm_caches_empty->("$desc: before set");
+
+        $cache->set( $key,  $value );
+        $cache->set( $key2, $value2 );
+        $confirm_caches_populated->("$desc: after set");
+        $remove_code->();
+
+        $confirm_caches_empty->("$desc: before set_multi");
+        $cache->set_multi( { $key => $value, $key2 => $value2 } );
+        $confirm_caches_populated->("$desc: after set_multi");
+        $remove_code->();
+
+        $confirm_caches_empty->("$desc: before return");
+    };
+
+    $confirm_caches_empty = sub {
+        my ($desc) = @_;
+        ok( !defined( $cache->get($key) ),
+            "primary cache is not populated with '$key' - $desc" );
+        ok( !defined( $subcache->get($key) ),
+            "subcache is not populated with '$key' - $desc" );
+        ok( !defined( $cache->get($key2) ),
+            "primary cache is not populated #2 with '$key2' - $desc" );
+        ok( !defined( $subcache->get($key2) ),
+            "subcache is not populated #2 with '$key2' - $desc" );
+    };
+
+    $confirm_caches_populated = sub {
+        my ($desc) = @_;
+        is( $cache->get($key), $value,
+            "primary cache is populated with '$key' - $desc" );
+        is( $subcache->get($key), $value,
+            "subcache is populated with '$key' - $desc" );
+        is( $cache->get($key2), $value2,
+            "primary cache is populated with '$key2' - $desc" );
+        is( $subcache->get($key2), $value2,
+            "subcache is populated with '$key2' - $desc" );
+    };
+
+    $test_remove_method->(
+        'remove', sub { $cache->remove($key); $cache->remove($key2) }
+    );
+    $test_remove_method->(
+        'expire', sub { $cache->expire($key); $cache->expire($key2) }
+    );
+    $test_remove_method->(
+        'expire_if',
+        sub {
+            $cache->expire_if( $key,  sub { 1 } );
+            $cache->expire_if( $key2, sub { 1 } );
+        }
+    );
+    $test_remove_method->( 'clear', sub { $cache->clear() } );
+}
+
 sub test_clear : Tests {
     my $self  = shift;
     my $cache = $self->{cache};
@@ -643,7 +983,7 @@ sub test_clear : Tests {
     }
 }
 
-sub test_logging : Test(8) {
+sub test_logging : Test(10) {
     my $self  = shift;
     my $cache = $self->{cache};
 
@@ -660,24 +1000,33 @@ sub test_logging : Test(8) {
       ( $driver eq 'Multilevel' ? 'MISS' : 'MISS \(expired\)' );
 
     my $start_time = time();
+
     $cache->get($key);
     $log->contains_ok(
-        qr/cache get for .* key='$key', driver='$driver': $miss_not_in_cache/);
+        qr/cache get for .* key='$key', cache='$driver': $miss_not_in_cache/);
+    $log->empty_ok();
+
     $cache->set( $key, $value, 80 );
     my $length = length($value);
     $log->contains_ok(
-        qr/cache set for .* key='$key', size=$length, expires='1m20s', driver='$driver'/
+        qr/cache set for .* key='$key', size=$length, expires='1m20s', cache='$driver'/
     );
+    $log->empty_ok();
+
     $cache->get($key);
-    $log->contains_ok(qr/cache get for .* key='$key', driver='$driver': HIT/);
+    $log->contains_ok(qr/cache get for .* key='$key', cache='$driver': HIT/);
+    $log->empty_ok();
+
     local $CHI::Driver::Test_Time = $start_time + 120;
     $cache->get($key);
     $log->contains_ok(
-        qr/cache get for .* key='$key', driver='$driver': $miss_expired/);
+        qr/cache get for .* key='$key', cache='$driver': $miss_expired/);
+    $log->empty_ok();
+
     $cache->remove($key);
     $cache->get($key);
     $log->contains_ok(
-        qr/cache get for .* key='$key', driver='$driver': $miss_not_in_cache/);
+        qr/cache get for .* key='$key', cache='$driver': $miss_not_in_cache/);
     $log->empty_ok();
 }
 
@@ -758,6 +1107,20 @@ sub test_obj_ref : Tests(8) {
     $validate_obj->($obj);
 }
 
+sub test_no_leak : Tests(2) {
+    my ($self) = @_;
+
+    my $weakref;
+    {
+        my $cache = $self->new_cache();
+        $weakref = $cache;
+        weaken($weakref);
+        ok( defined($weakref) && $weakref->isa('CHI::Driver'),
+            "weakref is defined" );
+    }
+    ok( !defined($weakref), "weakref is no longer defined - cache was freed" );
+}
+
 ## no critic
 { package My::CHI; use Mouse; extends 'CHI' }
 
